from models import *
from settings import DATA_DIR
import json
from os.path import join
import re
import datetime
from decimal import Decimal
from django.contrib.gis.geos import Point
import csv2kml

def import_stations():
    stations = json.load(open(join(DATA_DIR, "stations.json")))
    '''
    states = []
    for s in stations:
        if s['state'] not in states and s['state'] != None and s['state'].strip() != '':
            states.append(s['state'])
    for s in states:
        state = State(name=s)
        print state.name
        state.save()
    '''

    for s in stations:
        station = Station()
        station.data_id = s['id']
        station.code = s['code']
        station.name = s['name']
        station.zone = s['zone']
        if s['state'] is None:
            s['state'] = ''
        station.state = s['state']
        if s['address'] is None:
            s['address'] = ''
        station.address = s['address']
        print station.name
        station.save()


def import_trains():
    trains = json.load(open(join(DATA_DIR, "trains.json")))
    errors = []
    print(len(trains))
    for t in trains:
        train = Train()
        train.data_id = t['id']
        train.name = t['name']
        train.number = t['number']
        if t['return_train'] is None:
            t['return_train'] = ''
        train.return_train = t['return_train']
        duration_h = getHours(t['duration'])
        if duration_h is None:
            errors.append({
                'typ': 'Invalid Duration (H)',
                'train_data_id': t['id'],
                'duration': t['duration']
            })
            duration_h = 0
        train.duration_h = duration_h

        duration_m = getMinutes(t['duration'])
        if duration_m is None:
            errors.append({
                'typ': 'Invalid Duration (M)',
                'train_data_id': t['id'],
                'duration': t['duration']
            })
            duration_m = 0
        train.duration_m = duration_m

        train.zone = t['zone']
        train.date_from = getDate(t['date_from'])
        train.date_to = getDate(t['date_to'])
        try:
            train.from_station = Station.objects.get(code=t['from_station_code'])
        except:
            errors.append({
                'typ': 'From Station Not Found',
                'train_data_id': t['id'],
                'train_number': t['number'],
                'from_station_code': t['from_station_code'],
                'from_station_name': t['from_station_name']
            })
        try:
            train.to_station = Station.objects.get(code=t['to_station_code'])
        except:
            errors.append({
                'typ': 'To Station Not Found',
                'train_data_id': t['id'],
                'train_number': t['number'],
                'to_station_code': t['to_station_code'],
                'to_station_name': t['to_station_name']
            })

        train.number_of_halts = int(t['number_of_halts'])
        train.typ = t['type']
        departure = getTime(t['departure'])
        if departure is None:
            errors.append({
                'typ': 'Invalid Departure Time',
                'train_data_id': t['id'],
                'departure': t['departure']
            })
            departure = datetime.time(0,0)
        train.departure = departure

        arrival = getTime(t['arrival'])
        if departure is None:
            errors.append({
                'typ': 'Invalid Arrival Time',
                'train_data_id': t['id'],
                'arrival': t['arrival']
            })
            arrival = datetime.time(0,0)
        train.arrival = arrival
#        train.arrival = getTime(t['arrival'])
        try:
            train.distance = int(t['distance'].replace("km", "").strip())
        except:
            train.distance = 0
        train.departure_days = t['departure_days']

        boolFields = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday', 'chair_car', 'sleeper', 'first_class', 'third_ac', 'second_ac', 'first_ac']
        for b in boolFields:
            train.__setattr__(b, getBool(t[b]))
        print train.name
        train.save()
    errors_file = open(join(DATA_DIR, "trainErrors.json"), "w")
    json.dump(errors, errors_file, indent=2)
    errors_file.close()

timeRe = re.compile(r'([0-9]{1,2})h\s([0-9]{1,2})m?')

def getHours(s):
    try:
        return int(re.findall(timeRe,s)[0][0])
    except:
        return None

def getMinutes(s):
    try:
        return int(re.findall(timeRe,s)[0][1])
    except:
        return None

timeRe2 = re.compile(r'([0-9]{1,2})\:([0-9]{1,2})')
def getTime(s):
    try:
        hrs = int(re.findall(timeRe2,s)[0][0])
        mins = int(re.findall(timeRe2,s)[0][1])
        return datetime.time(hrs, mins)
    except:
        return None

dateRe = re.compile(r'([a-zA-Z]{3})\s([0-9]{1,2})')
months = ['', 'jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
def getDate(s):
    if s is None or s == '':
        return ''
    s = s.strip()
    matches = re.findall(dateRe, s)
    monthStr = matches[0][0]
    monthInt = months.index(monthStr.lower())
    date = int(matches[0][1])    
    return "%02d-%02d" % (monthInt, date,)

def getMins(s):
    if s is None:
        return None
    s = s.replace('m', '').strip()
    if s == '':
        return None
    else:
        return s

def getBool(s):
    if s == None or int(s) == 0:
        return False
    else:
        return True

def import_schedule():
    schedules = json.load(open(join(DATA_DIR, "schedule.json")))
    errors = []
    for s in schedules:
        schedule = Schedule()
        schedule.data_id = s['id']
        schedule.arrival = getTime(s['arrival'])
        schedule.departure = getTime(s['departure'])
        schedule.halt = getMins(s['halt'])
        try:
            schedule.stop_number = Decimal(s['stop_number'])
        except:
            errors.append({
                'typ': 'Invalid Stop Number',
                'data_id': s['id'],
                'stop_number': s['stop_number']
            })
            schedule.stop_number = Decimal('0')
        try:
            schedule.station = Station.objects.get(code=s['station_code'])
        except:
            schedule.station = Station.objects.filter(code=s['station_code'])[0]
            errors.append({
                'typ': 'Invalid or non-unique station code (NOT IMPORTED)',
                'data_id': s['id'],
                'code': s['station_code']
            })
            continue
        try:
            schedule.train = Train.objects.get(number=s['train_number'])
        except:
            schedule.train = Train.objects.filter(number=s['train_number'])[0]
            errors.append({
                'typ': 'Invalid or non-unique train code (NOT IMPORTED)',
                'data_id': s['id'],
                'train_number': s['train_number']
            })
            continue
        try:
            schedule.day = int(s['day'])
        except:
            errors.append({
                'typ': 'Day Error',
                'data_id': s['id'],
                'day': s['day']
            })
            schedule.day = 0
    
        try:
            schedule.distance_travelled = int(s['distance_travelled'])
        except:
            errors.append({
                'typ': 'Day Error',
                'data_id': s['id'],
                'distance': s['distance_travelled']
            })
            schedule.distance_travelled = 0
        try:
            schedule.save()
        except:
            errors.append({
                'typ': 'Something foobarred on Save :(',
                'data_id': s['id']
            })
    errorsFile = open(join(DATA_DIR, "scheduleErrors.json"), "w")
    json.dump(errors, errorsFile, indent=2)
    errorsFile.close()

def geolocate_stations():
    errors = []
    for s in Station.objects.filter(point=None)[0:1000]:
        coords = csv2kml.get_coordinates(s.address)
        if coords is not None:
            pt = Point(coords[0], coords[1])
            s.point = pt
            print s.name
            s.save()
        else:
            errors.append({
                'typ': 'Failed to geolocate',
                'data_id': s.data_id,
                'address': s.address
            })
    errorsFile = open(join(DATA_DIR, "geolocateErrors.json"), "w")
    json.dump(errors, errorsFile, indent=2)
    errorsFile.close()

def import_all():
    import_stations()
    import_trains()
    import_schedules()

